#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_SoftwareIdentityFileCheck`` provider.
"""

import shutil
import os
import pywbem
import re
import stat
import subprocess
import unittest

from lmi.test.lmibase import enable_lmi_exceptions
import package
import swbase
import util

RE_CHECKSUM = re.compile(r'^([0-9a-fA-F]+)\s+.*')

SOFTWARE_ELEMENT_STATE_EXECUTABLE = 2
FILE_TYPE_UNKNOWN                 = 0
FILE_TYPE_FILE                    = 1
FILE_TYPE_DIRECTORY               = 2
FILE_TYPE_SYMLINK                 = 3
FILE_TYPE_FIFO                    = 4
FILE_TYPE_CHARDEV                 = 5
FILE_TYPE_BLOCKDEV                = 6
FAILED_FLAGS_EXISTENCE            = 0
FAILED_FLAGS_SIZE                 = 1
FAILED_FLAGS_MODE                 = 2
FAILED_FLAGS_CHECKSUM             = 3
FAILED_FLAGS_DEVNUM               = 4
FAILED_FLAGS_TARGET               = 5
FAILED_FLAGS_UID                  = 6
FAILED_FLAGS_GID                  = 7
FAILED_FLAGS_MTIME                = 8
FILE_MODE_XOTH                    = 0
FILE_MODE_WOTH                    = 1
FILE_MODE_ROTH                    = 2
FILE_MODE_XGRP                    = 3
FILE_MODE_WGRP                    = 4
FILE_MODE_RGRP                    = 5
FILE_MODE_XUSR                    = 6
FILE_MODE_WUSR                    = 7
FILE_MODE_RUSR                    = 8
FILE_MODE_SVTX                    = 9 # sticky bit
FILE_MODE_SGID                    = 10
FILE_MODE_SUID                    = 11
INVOKE_SATISFIED                  = 0
INVOKE_NOT_SATISFIED              = 2

# maps algorithm id to the shell command
HASH_COMMAND = {
        1  : "md5sum",
        2  : "sha1sum",
        8  : "sha256sum",
        9  : "sha384sum",
        10 : "sha512sum",
        11 : "sha224sum"
}

# maps algorithm id to the length of digest string
HASH_DIGEST_LENGTH = {
        1  : 32,    #MD5
        2  : 40,    #SHA1
        8  : 64,    #SHA256
        9  : 96,    #SHA384
        10 : 128,   #SHA512
        11 : 56     #SHA224
}

class TestSoftwareIdentityFileCheck(swbase.SwTestCase):
    """
    Basic cim operations test.
    """

    CLASS_NAME = "LMI_SoftwareIdentityFileCheck"
    KEYS = ("CheckID", "Name", "SoftwareElementID", "SoftwareElementState")

    def make_checksum_str(self, csumnum, filename):
        """
        :returns: Checksum of installed file.
        :rtype: string
        """
        return RE_CHECKSUM.match(subprocess.check_output([
            HASH_COMMAND[csumnum], filename])).group(1).lower()

    def make_op(self, pkg, file_name):
        """
        :returns: Object path of ``LMI_SoftwareIdentityFileCheck``
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return self.cim_class.new_instance_name({
            "CheckID" : 'LMI:LMI_SoftwareIdentityFileCheck',
            "Name" : file_name,
            "SoftwareElementID" : pkg.nevra,
            "SoftwareElementState" : pywbem.Uint16(
                SOFTWARE_ELEMENT_STATE_EXECUTABLE),
            "TargetOperatingSystem" : util.get_target_operating_system(),
            "Version" : pkg.evra
        })

    def setUp(self):
        to_uninstall = set()
        for repo in self.repodb.values():
            for pkg in repo.packages:
                to_uninstall.add(pkg.name)
        to_uninstall = list(package.filter_installed_packages(to_uninstall))
        package.remove_pkgs(to_uninstall, suppress_stderr=True)

    def do_check_symlink(self, pkg, filepath, inst):
        """
        Check symbolink link.
        """
        target = os.readlink(filepath)
        stats = os.lstat(filepath)

        self.assertEqual(inst.FileType, pywbem.Uint16(FILE_TYPE_SYMLINK),
                "Unexpected file type of symlink for %s:%s"
                % (pkg.name, filepath))
        self.assertEqual(inst.UserID, stats.st_uid,
                "Unexpected uid of symlink for %s:%s"  % (pkg.name, filepath))
        self.assertEqual(inst.GroupID, stats.st_gid,
                "Unexpected gid of symlink for %s:%s"  % (pkg.name, filepath))
        self.assertEqual(inst.FileMode, stats.st_mode,
                "Unexpected mode of symlink for %s:%s" % (pkg.name, filepath))
        self.assertEqual(inst.FileSize, stats.st_size,
                "Unexpected size of symlink for %s:%s" % (pkg.name, filepath))
        self.assertEqual(inst.LinkTarget, target,
                "Unexpected target of symlink for %s:%s" % (pkg.name, filepath))
        self.assertEqual(inst.LastModificationTime, int(stats.st_mtime),
                "Unexpected mtime of symlink for %s:%s"  % (pkg.name, filepath))
        self.assertIsNone(inst.Checksum,
                "Checksum should be None for symlink %s:%s"
                % (pkg.name, filepath))
        self.assertIsNone(inst.FileChecksum,
                "FileChecksum should be None for symlink %s:%s"
                % (pkg.name, filepath))
        self.assertIsNone(inst.MD5Checksum,
                "MD5Checksum should be None for symlink %s:%s"
                % (pkg.name, filepath))

        # modify owner
        prev_user = inst.UserID
        os.lchown(filepath, stats.st_uid + 1, -1)
        inst.refresh()
        self.assertEqual(inst.UserID, inst.UserIDOriginal + 1,
            "Unexpected uid of modified symlink for %s:%s"%(
            pkg.name, filepath))
        self.assertEqual(inst.UserID, prev_user + 1,
            "Unexpected uid of modified symlink for %s:%s"%(
            pkg.name, filepath))
        self.assertEqual(inst.GroupID, stats.st_gid,
            "Unexpected gid of modified symlink for %s:%s"%(
            pkg.name, filepath))
        self.assertEqual(inst.FailedFlags, [FAILED_FLAGS_UID])

        # modify link_target
        os.remove(filepath)
        lt_modif = "wrong" + "*"*len(inst.LinkTargetOriginal)
        os.symlink(lt_modif, filepath)
        os.lchown(filepath, inst.UserIDOriginal, inst.GroupIDOriginal)

        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_TARGET]))
        self.assertGreater(len(inst.LinkTarget), len(inst.LinkTargetOriginal))

        self.assertTrue(inst.FileExists,
            "File %s:%s should exist"%(pkg.name, filepath))
        self.assertEqual(inst.FileType, inst.FileTypeOriginal,
            "File type should match for symlink %s:%s"%(pkg.name, filepath))
        self.assertNotEqual(inst.FileSizeOriginal, inst.FileSize,
            "File size should not match for symlink %s:%s"%(
                pkg.name, filepath))
        self.assertEqual(inst.FileMode, inst.FileModeOriginal,
            "File mode should match for symlink %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.LinkTarget, lt_modif,
            "Link target should match modified path %s:%s"%(
                pkg.name, filepath))
        self.assertNotEqual(inst.LinkTargetOriginal, inst.LinkTarget,
            "Link target should not match for symlink %s:%s"%(
                pkg.name, filepath))
        self.assertEqual(inst.UserID, inst.UserIDOriginal,
            "File uid should match for symlink %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.GroupID, inst.GroupIDOriginal,
            "File gid should match for symlink %s:%s"%(pkg.name, filepath))

    def do_check_directory(self, pkg, filepath, inst):
        """
        Check directory.
        """
        stats = os.lstat(filepath)

        self.assertEqual(inst.FileType, pywbem.Uint16(FILE_TYPE_DIRECTORY),
                "Unexpected type for directory %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.UserID, stats.st_uid,
                "Unexpected uid for directory %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.GroupID, stats.st_gid,
                "Unexpected gid for directory %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.FileMode, stats.st_mode,
                "Unexpected mode for directory %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.FileSize, stats.st_size,
                "Unexpected size for directory %s:%s"%(pkg.name, filepath))
        self.assertIs(inst.LinkTarget, None)
        self.assertIsNone(inst.FileChecksum,
                "Unexpected checksum for directory %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.LastModificationTime, int(stats.st_mtime),
                "Unexpected mtime for directory %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.FailedFlags, [])

    def do_check_file(self, pkg, filepath, inst):
        """
        Check regular file.
        """
        stats = os.lstat(filepath)

        self.assertEqual(inst.FileType, pywbem.Uint16(FILE_TYPE_FILE),
            "Unexpected file type for %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.UserID, stats.st_uid,
            "Unexpected file uid for %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.GroupID, stats.st_gid,
            "Unexpected gid for regular file %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.FileMode, stats.st_mode,
            "Unexpected mode for regular file %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.FileSize, stats.st_size,
            "Unexpected size for regular file %s:%s"%(pkg.name, filepath))
        self.assertIs(inst.LinkTarget, None)
        csum = self.make_checksum_str(inst.ChecksumType, filepath)
        self.assertEqual(inst.FileChecksum.lower(), csum,
            "Unexpected checksum for regular file %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.LastModificationTime,
                inst.LastModificationTimeOriginal,
                "Unexpected mtime for regular file %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.LastModificationTime, int(stats.st_mtime),
                "Unexpected mtime for regular file %s:%s"%(pkg.name, filepath))

        # make it longer
        with open(filepath, "a+") as fobj:
            fobj.write("data\n")
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([
                FAILED_FLAGS_SIZE,
                FAILED_FLAGS_CHECKSUM,
                FAILED_FLAGS_MTIME,
            ]))

        self.assertGreater(inst.FileSize, inst.FileSizeOriginal,
                "File size should be greater, then expected for regular file"
                " %s:%s"%(pkg.name, filepath))
        self.assertGreater(inst.LastModificationTime,
                inst.LastModificationTimeOriginal,
                "Unexpected mtime for regular file %s:%s"%(pkg.name, filepath))

        self.assertTrue(inst.FileExists,
            "Regular file should exist %s:%s"%(pkg.name, filepath))

        # change file type
        os.remove(filepath)
        os.symlink(filepath, filepath)
        os.lchown(filepath, inst.UserIDOriginal,
                inst.GroupIDOriginal)
        inst.refresh()
        self.assertNotEqual(inst.LinkTargetOriginal, inst.LinkTarget,
                "Link target should not match for %s:%s"%(pkg.name, filepath))
        self.assertNotEqual(inst.FileSizeOriginal, inst.FileSize,
                "File size should not match for %s:%s"%(pkg.name, filepath))
        self.assertGreater(inst.LastModificationTime,
                inst.LastModificationTimeOriginal,
                "File mtime should be greater than expected for %s:%s"%(
                    pkg.name, filepath))
        self.assertNotEqual(inst.FileTypeOriginal, inst.FileType,
                "File type should not match for %s:%s"%(pkg.name, filepath))
        self.assertEqual(inst.FileType, pywbem.Uint16(FILE_TYPE_SYMLINK),
                "File type should match for %s:%s"%(pkg.name, filepath))
        self.assertIn(pywbem.Uint16(FAILED_FLAGS_MODE), inst.FailedFlags)

        # remove it
        os.remove(filepath)
        inst.refresh()
        self.assertEqual(inst.LinkTarget, inst.LinkTargetOriginal,
                "Link target does not match for regular file %s:%s"%(
                    pkg.name, filepath))
        self.assertNotEqual(inst.FileSizeOriginal, inst.FileSize,
                "File size should not match for regular file %s:%s"%(
                    pkg.name, filepath))
        self.assertIsNone(inst.LastModificationTime)
        self.assertIsNone(inst.FileType)
        self.assertIsNone(inst.FileChecksum)
        self.assertIsNone(inst.FileMode)
        self.assertIsNone(inst.UserID)
        self.assertIsNone(inst.GroupID)
        self.assertFalse(inst.FileExists)
        self.assertEqual(inst.FailedFlags, [FAILED_FLAGS_EXISTENCE])

    def do_check_dev(self, pkg, filepath, inst):
        """
        Check device file.
        """
        stats = os.lstat(filepath)

        is_block = stat.S_ISBLK(stats.st_mode)
        self.assertEqual(inst.FileType, pywbem.Uint16(
            FILE_TYPE_BLOCKDEV if is_block else FILE_TYPE_CHARDEV),
                "Unexpected file type of dev file for %s:%s"
                % (pkg.name, filepath))
        self.assertEqual(inst.UserID, stats.st_uid,
                "Unexpected uid of dev file for %s:%s"  % (pkg.name, filepath))
        self.assertEqual(inst.GroupID, stats.st_gid,
                "Unexpected gid of dev file for %s:%s"  % (pkg.name, filepath))
        self.assertEqual(inst.FileMode, stats.st_mode,
                "Unexpected mode of dev file for %s:%s" % (pkg.name, filepath))
        self.assertEqual(inst.FileSize, stats.st_size,
                "Unexpected size of dev file for %s:%s" % (pkg.name, filepath))
        self.assertEqual(inst.LinkTarget, None)
        self.assertEqual(inst.LastModificationTime, int(stats.st_mtime),
                "Unexpected mtime of dev file for %s:%s" % (pkg.name, filepath))
        self.assertIsNone(inst.Checksum,
                "Checksum should be None for dev file %s:%s"
                % (pkg.name, filepath))
        self.assertIsNone(inst.FileChecksum,
                "FileChecksum should be None for dev file %s:%s"
                % (pkg.name, filepath))
        self.assertIsNone(inst.MD5Checksum,
                "MD5Checksum should be None for dev file %s:%s"
                % (pkg.name, filepath))

        os.remove(filepath)
        subprocess.call(['/usr/bin/mknod', filepath, 'b' if is_block else 'c',
            str(os.major(stats.st_rdev) + 1), str(os.minor(stats.st_rdev) + 1)])
        self.assertTrue(os.path.exists(filepath))
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_DEVNUM]))
        self.assertEqual(inst.FileMode, stats.st_mode)

    def check_filepath(self, pkg, filepath):
        """
        Make a check of particular file of package.
        All files are expected to have no flaw.
        """
        objpath = self.make_op(pkg, filepath)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        self.assertCIMNameEqual(inst.path, objpath,
            msg="Object paths of instance must match for %s:%s"%(
            pkg.name, filepath))
        for key in self.KEYS:
            if key.lower() == "targetoperatingsystem":
                self.assertIsInstance(getattr(objpath, key), (int, long))
            else:
                self.assertEqual(getattr(objpath, key), getattr(inst, key),
                    "OP key %s values should match for %s:%s"%(
                    key, pkg.name, filepath))

        self.assertTrue(inst.FileExists,
            "File %s:%s must exist"%(pkg.name, filepath))
        self.assertEqual(len(inst.FailedFlags), 0,
                "FailedFlags must be empty for %s:%s, not: %s" % (
                    str(pkg), filepath, inst.FailedFlags))

        for prop in ( "FileType", "UserID", "GroupID"
                    , "FileMode", "FileSize", "LinkTarget"
                    , "FileChecksum", "FileModeFlags"):
            if (  (  (  os.path.islink(filepath)
                     or (not os.path.isfile(filepath)))
                  and prop == "FileSize")
               or (os.path.islink(filepath) and prop == "FileMode")):
                continue
            self.assertEqual(
                    getattr(inst, prop+"Original"), getattr(inst, prop),
                "%s should match for %s:%s"%(prop, pkg.name, filepath))
        if os.path.islink(filepath):
            self.do_check_symlink(pkg, filepath, inst)
        elif os.path.isdir(filepath):
            self.do_check_directory(pkg, filepath, inst)
        elif os.path.isfile(filepath):
            self.do_check_file(pkg, filepath, inst)

    @swbase.test_with_packages('stable#pkg1')
    def test_get_directory(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityFileCheck`` with
        directory.
        """
        pkg = self.get_repo('stable')['pkg1']
        filepath = '/usr/share/openlmi-sw-test-pkg1'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.to_instance()
        self.assertEqual(inst.FileType, FILE_TYPE_DIRECTORY)
        self.check_filepath(pkg, filepath)

    @swbase.test_with_packages('stable#pkg1')
    def test_check_config(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityFileCheck`` with
        config file..
        """
        pkg = self.get_repo('stable')['pkg1']
        filepath = '/etc/openlmi/software/test/dummy_config.cfg'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.to_instance()
        self.assertEqual(inst.FileType, FILE_TYPE_FILE)
        self.check_filepath(pkg, filepath)

    @enable_lmi_exceptions
    @swbase.test_with_packages('stable#pkg1', 'stable#pkg2')
    def test_get_instance_invalid(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityFileCheck`` with
        file not belonging to particular package.
        """
        stable = self.get_repo('stable')
        pkg1 = stable['pkg1']
        pkg2 = stable['pkg2']
        fp1 = '/etc/openlmi/software/test/dummy_config.cfg'
        fp2 = '/usr/share/openlmi-sw-test-pkg2'
        objpath1 = self.make_op(pkg1, fp2)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath1.to_instance)
        objpath2 = self.make_op(pkg2, fp1)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath2.to_instance)
        objpath1.wrapped_object["Name"] = fp1
        self.assertNotEqual(objpath1.to_instance(), None)
        objpath2.wrapped_object["Name"] = fp2
        self.assertNotEqual(objpath2.to_instance(), None)

    @swbase.test_with_packages('stable#pkg2')
    def test_check_symlink(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityFileCheck`` with
        symbolic link.
        """
        pkg = self.get_repo('stable')['pkg2']
        filepath = '/usr/share/openlmi-sw-test-pkg2/symlinks/absolute'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.to_instance()
        self.assertEqual(inst.FileType, FILE_TYPE_SYMLINK)
        self.check_filepath(pkg, filepath)

    @swbase.test_with_packages('stable#pkg3')
    def test_check_chardev(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityFileCheck`` with
        block and character device files.
        """
        pkg = self.get_repo('stable')['pkg3']
        filepath = '/usr/share/openlmi-sw-test-pkg3/devs/char12'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        self.assertEqual(inst.FileType, FILE_TYPE_CHARDEV)
        self.check_filepath(pkg, filepath)
        self.do_check_dev(pkg, filepath, inst)

    @swbase.test_with_packages('stable#pkg3')
    def test_check_blockdev(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityFileCheck`` with
        block and character device files.
        """
        pkg = self.get_repo('stable')['pkg3']
        filepath = '/usr/share/openlmi-sw-test-pkg3/devs/block56'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        self.assertEqual(inst.FileType, FILE_TYPE_BLOCKDEV)
        self.check_filepath(pkg, filepath)
        self.do_check_dev(pkg, filepath, inst)

    @swbase.test_with_packages('stable#pkg4')
    def test_permissions(self):
        """
        Test ``FileMode`` property of ``LMI_SoftwareIdentityFileCheck``.
        """
        pkg = self.get_repo('stable')['pkg4']
        permsdir = '/usr/share/openlmi-sw-test-pkg4/perms'
        objpath = self.make_op(pkg, permsdir)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        permsdir += '/'
        self.assertEqual(inst.FileMode, stat.S_IFDIR | stat.S_ISVTX | 0777)
        self.assertEqual(set(inst.FileModeFlags), set(range(10)))

        objpath.wrapped_object["Name"] = permsdir + 'rwxrwxrwx'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 0777)
        self.assertEqual(set(inst.FileModeFlags), set(range(9)))

        objpath.wrapped_object["Name"] = permsdir + 'rw-rw-rw-'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 0666)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_ROTH, FILE_MODE_RGRP, FILE_MODE_RUSR,
            FILE_MODE_WOTH, FILE_MODE_WGRP, FILE_MODE_WUSR]))

        objpath.wrapped_object["Name"] = permsdir + 'r--r--r--'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 0444)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_ROTH, FILE_MODE_RGRP, FILE_MODE_RUSR]))

        objpath.wrapped_object["Name"] = permsdir + 'r-xr-xr-x'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 0555)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_ROTH, FILE_MODE_RGRP, FILE_MODE_RUSR,
            FILE_MODE_XOTH, FILE_MODE_XGRP, FILE_MODE_XUSR]))

        objpath.wrapped_object["Name"] = permsdir + 'r---w---x'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 0421)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_RUSR, FILE_MODE_WGRP, FILE_MODE_XOTH]))

        objpath.wrapped_object["Name"] = permsdir + 'rwSr--r--'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 04644)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_RUSR, FILE_MODE_WUSR,
            FILE_MODE_RGRP, FILE_MODE_ROTH, FILE_MODE_SUID]))

        objpath.wrapped_object["Name"] = permsdir + 'rw-r-Sr--'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 02644)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_RUSR, FILE_MODE_WUSR,
            FILE_MODE_RGRP, FILE_MODE_ROTH, FILE_MODE_SGID]))

        objpath.wrapped_object["Name"] = permsdir + 'rw-r--r-T'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 01644)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_RUSR, FILE_MODE_WUSR,
            FILE_MODE_RGRP, FILE_MODE_ROTH, FILE_MODE_SVTX]))

        objpath.wrapped_object["Name"] = permsdir + 'rwsr-sr-t'
        inst = objpath.to_instance()
        self.assertEqual(inst.FileMode, stat.S_IFREG | 07755)
        self.assertEqual(set(inst.FileModeFlags), set([
            FILE_MODE_ROTH, FILE_MODE_RGRP, FILE_MODE_RUSR,
            FILE_MODE_XOTH, FILE_MODE_XGRP, FILE_MODE_XUSR,
            FILE_MODE_WUSR, FILE_MODE_SUID, FILE_MODE_SGID, FILE_MODE_SVTX]))

    @enable_lmi_exceptions
    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames`` call on ``LMI_SoftwareIdentityFileCheck``
        that should not be supported.
        """
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_SUPPORTED,
                self.cim_class.instance_names)

    @swbase.test_with_packages('stable#pkg1')
    def test_method_invoke_on_file(self):
        """
        Test ``Invoke`` method of ``LMI_SoftwareIdentityFileCheck`` on regular
        file.
        """
        pkg = self.get_repo('stable')['pkg1']
        fp = '/usr/share/openlmi-sw-test-pkg1/README'
        objpath = self.make_op(pkg, fp)
        inst = objpath.to_instance()
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED,
                "Invoke method should be successful for %s:%s"  % (pkg, fp))
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED,
                "InvokeOnSystem method should be successful for %s:%s"
                % (pkg.name, fp))

        stats = os.stat(fp)
        backup = fp + '.bak'
        shutil.copy(fp, backup)

        # change mode
        os.chmod(fp, 0600)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_MODE]))

        # restore it
        shutil.copy(backup, fp)
        os.utime(fp, (stats.st_atime, stats.st_mtime))
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # modify contents
        with open(fp, 'a') as fobj:
            fobj.write('new line\n')
        os.utime(fp, (stats.st_atime, stats.st_mtime))
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags),
                set([FAILED_FLAGS_SIZE, FAILED_FLAGS_CHECKSUM]))

        # restore it
        shutil.copy(backup, fp)
        os.utime(fp, (stats.st_atime, stats.st_mtime))
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # touch it
        os.utime(fp, None)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_MTIME]))

    @enable_lmi_exceptions
    @swbase.test_with_packages('stable#pkg1')
    def test_method_invoke_on_uninstalled_package(self):
        """
        Test ``Invoke`` method of ``LMI_SoftwareIdentityFileCheck`` on
        uninstalled package.
        """
        pkg = self.get_repo('stable')['pkg1']
        fp = '/usr/share/openlmi-sw-test-pkg1/README'
        objpath = self.make_op(pkg, fp)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        package.remove_pkgs(pkg.name)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, inst.Invoke)

    @swbase.test_with_packages('stable#pkg2')
    def test_method_invoke_on_symlink(self):
        """
        Test ``Invoke`` method of ``LMI_SoftwareIdentityFileCheck`` on symbolic
        link.
        """
        pkg = self.get_repo('stable')['pkg2']
        fp = '/usr/share/openlmi-sw-test-pkg2/symlinks/relative'
        objpath = self.make_op(pkg, fp)
        inst = objpath.to_instance()
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED,
                "Invoke method should be successful for %s:%s"  % (pkg, fp))
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED,
                "InvokeOnSystem method should be successful for %s:%s"
                % (pkg.name, fp))
        stats = os.stat(fp)
        backup = fp + '.bak'
        shutil.copy(fp, backup)

        # touch it (mtime)
        os.utime(fp, None)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # change owner
        os.lchown(fp, 1, -1)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_UID]))

        # remove symlink
        os.remove(fp)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_EXISTENCE]))

        # restore it (mtime)
        os.symlink("../data/target.txt", fp)
        os.utime(fp, (stats.st_atime, stats.st_mtime))
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # point to different target
        os.remove(fp)
        os.symlink("../data/target.xtx", fp)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_TARGET]))

    @swbase.test_with_packages('stable#pkg3')
    def test_method_invoke_on_fifo(self):
        """
        Test ``Invoke`` method of ``LMI_SoftwareIdentityFileCheck`` on fifo.
        """
        pkg = self.get_repo('stable')['pkg3']
        fp = '/usr/share/openlmi-sw-test-pkg3/fifos/pipe'
        objpath = self.make_op(pkg, fp)
        inst = objpath.to_instance()
        self.assertEqual(inst.FileType, FILE_TYPE_FIFO)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED,
                "Invoke method should be successful for %s:%s"  % (pkg, fp))
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED,
                "InvokeOnSystem method should be successful for %s:%s"
                % (pkg.name, fp))
        stats = os.stat(fp)

        # touch it (mtime)
        os.utime(fp, None)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # change owner
        os.chown(fp, 1, -1)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_UID]))

        # remove fifo
        os.remove(fp)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_EXISTENCE]))

        # restore it
        os.mkfifo(fp, 0644)
        os.utime(fp, (stats.st_atime, stats.st_mtime))
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # change mode
        os.chmod(fp, 0400)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_MODE]))

        # replace it with regular file
        os.remove(fp)
        with open(fp, 'w') as fobj:
            fobj.write('dummy file')
        os.utime(fp, (stats.st_atime, stats.st_mtime))
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(inst.FileType, FILE_TYPE_FILE)
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_MODE]))

    @swbase.test_with_packages('stable#pkg4')
    def test_method_invoke_on_directory(self):
        """
        Test ``Invoke`` method of ``LMI_SoftwareIdentityFileCheck``
        on directory.
        """
        pkg = self.get_repo('stable')['pkg4']
        fp = '/usr/share/openlmi-sw-test-pkg4/perms'
        objpath = self.make_op(pkg, fp)
        inst = objpath.to_instance()
        self.assertEqual(inst.FileType, FILE_TYPE_DIRECTORY)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED,
                "Invoke method should be successful for %s:%s"  % (pkg, fp))
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED,
                "InvokeOnSystem method should be successful for %s:%s"
                % (pkg.name, fp))

        # touch it (mtime)
        os.utime(fp, None)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # remove child
        os.remove(os.path.join(fp, 'rwxrwxrwx'))
        os.utime(fp, None)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_SATISFIED)

        # chmod
        os.chmod(fp, 0755)
        (rval, _, _) = inst.Invoke()
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        (rval, _, _) = inst.InvokeOnSystem(TargetSystem=self.system_iname)
        self.assertEqual(rval, INVOKE_NOT_SATISFIED)
        inst.refresh()
        self.assertEqual(set(inst.FailedFlags), set([FAILED_FLAGS_MODE]))

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestSoftwareIdentityFileCheck)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
#

import os;
import pywbem;
import optparse
import re
import cgi
import sys

class DotExporter(object):
    
    def __init__(self, cliconn):
        self.classcache = {}
        self.cliconn = cliconn
        # original list of classes
        self.classes = set()
        self.file = sys.stdout

    def load_class(self, classname):
        if classname in self.classcache:
            return self.classcache[classname]

        c = self.cliconn.GetClass(classname)
        self.classcache[classname] = c
        return c

    def display_type(self, param):
        """
            Return displayable type of given parameter.
            It adds [] if it's array and class name of referenced classes.
        """
        if param.reference_class:
            ptype = param.reference_class
        else:
            ptype = param.type
        if param.is_array:
            ptype = ptype + "[]"
        return ptype

    def print_parameters(self, params):
        """
            Print table of method parametes.
        """
        if not params:
            print >>self.file, "None"
            return

        print >>self.file, "<table class=\"parameters\">"
        for p in params.values():
            direction = set()
            if p.qualifiers.has_key("Out"):
                direction.add("OUT")
            if p.qualifiers.has_key("In"):
                direction.add("IN")
            if not direction:
                direction.add("IN")
            direction = "/".join(sorted(direction))

            print >>self.file, "<tr>"
            print >>self.file, "<td>%s</td><td>%s</td><td><b>%s</b></td>" % (direction, self.display_type(p), p.name)
            print >>self.file, "<td><table class=\"qualifiers\">"
            self.print_qualifiers(p.qualifiers.values())
            print >>self.file, "</table></td>"
            print >>self.file, "</tr>"
        print >>self.file, "</table>"

    def print_qualifiers(self, qualifiers):
        """
            Print contenf of table of qualifiers.
            Only Deprecated, Description, Values and ValueMap are recognized.
        """
        deprecated = ""
        for q in sorted(qualifiers):
            if q.name == "Deprecated":
                deprecated = "<div class=\"deprecated\">DEPRECATED</div> "
            if q.name == "Description":
                print >>self.file, "<tr><td class=\"qualifiers\" colspan=\"2\">%s%s</td></tr>" % (deprecated, self.escape(q.value))
            if q.name == "Values":
                print >>self.file, "<tr><td class=\"qualifiers\"><b>Values</b></td> <td class=\"qualifiers\"><table><tr><td class=\"qualifiers\">%s</td></tr></table></td></tr>" % ("</td></tr><tr><td class=\"qualifiers\">".join(q.value))
            if q.name == "ValueMap":
                print >>self.file, "<tr><td class=\"qualifiers\"><b>ValueMap</b></td> <td class=\"qualifiers\"><table><tr><td class=\"qualifiers\">%s</td></tr></table></td></tr>" % ("</td></tr><tr><td class=\"qualifiers\">".join(q.value))

    def compare_properties(self, p1, p2):
        """
            Compare two properties if they should printed in Inherited properties.
            Only Name, Description and Implemented are checked.
            Returns False, if the property should be printed in Local.
        """
        if p1.name != p2.name:
            return False
        d1 = p1.qualifiers.get("Description", None)
        d2 = p2.qualifiers.get("Description", None)
        if d1.value != d2.value:
            return False
        i1 = p1.qualifiers.get("Implemented", None)
        i2 = p2.qualifiers.get("Implemented", None)
        if i1 and i1.value and not (i2 and i2.value):
            return False
        return True

    def print_class(self, c, display_local = True, box_only = False):
        """
            Print one class, inc. header.
        """
        parent = None
        if c.superclass:
            parent = self.load_class(c.superclass)

        if c.superclass:
            # draw arrow to parent
            print >>self.file, "\"%s\"->\"%s\"" % (c.classname, c.superclass)

        if box_only:
            return

        local_props = []
        for name in sorted(c.properties.keys()):
            if parent and parent.properties.has_key(name):
                if not self.compare_properties(c.properties[name], parent.properties[name]):
                    # the property was overridden
                    local_props.append(c.properties[name])
            else:
                local_props.append(c.properties[name])

        local_methods = []
        for name in sorted(c.methods.keys()):
            if parent and parent.methods.has_key(name):
                if not self.compare_properties(c.methods[name], parent.methods[name]):
                    # the property was overridden
                    local_methods.append(c.methods[name])
            else:
                local_methods.append(c.methods[name])

        self.file.write("\"%s\" [href=\"%s.html\",shape=\"record\", label=\"{%s" %(c.classname, c.classname, c.classname))

        if display_local:
            self.file.write("|")
            for prop in local_props:
                self.file.write("%s %s\\l" % (self.display_type(prop), prop.name))

            self.file.write("|")

            if local_methods:
                for m in local_methods:
                    self.file.write("%s()\l" % (m.name))

        self.file.write("}\"];\n")


    def add_class(self, classname):
        self.classes.add(classname)

    def export(self, shrink, noassoc = False):
        """
            Print all classes and their parents.
        """
        print >>self.file, """
digraph "classes_No_Name" {
charset="utf-8"
rankdir=BT
node [shape="record" fontsize=10 fontname="sans-serif"]
edge [arrowhead="empty" fontsize=10 fontname="sans-serif"]
"""
        while self.classes:
            c = self.classes.pop()
            
            cl = self.load_class(c)
            if noassoc and cl.qualifiers.get("Association", False):
                continue
         
            if shrink and shrink.match(c):
            
                self.print_class(cl, box_only = True)
            else:
                self.print_class(cl)

        print >>self.file, "}"

description = """
Generate UML image for given classes. The tool connects to specified CIMOM
and reads class definition from there. Each class specified on command line
will be drawn as one box, containing locally defined or re-defined properties
and methods. Inheritance will be shown as arrow between a parent class and a
subclass.
"""

parser = optparse.OptionParser(usage="usage: %prog [options] classname [classname ...]", description=description)
parser.add_option('-u', '--url', action='store', dest='addr', default='https://localhost:5989', help='URL of CIM server, default: https://localhost:5989')
parser.add_option('-U', '--user', action='store', dest='user', default=None, help='CIM user name')
parser.add_option('-s', '--shrink', action='store', dest='shrink', default=None, help='Regular expression pattern of CIM classes, which will be drawn only as boxes, without properties.')
parser.add_option('-A', '--no-associations', action='store_true', dest='noassoc', default=False, help='Skip association classes.')
parser.add_option('-P', '--password', action='store', dest='password', default=None, help='CIM password')
(options, args) = parser.parse_args()

sys.stdout.softspace=0

shrink = None
if options.shrink:
    shrink = re.compile(options.shrink)

cliconn = pywbem.WBEMConnection(options.addr, (options.user, options.password))
exporter = DotExporter(cliconn)
for c in args:
    exporter.add_class(c)
exporter.export(shrink = shrink, noassoc = options.noassoc)




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#

"""Common utilities for LMI_Software* providers
"""

import platform
import pywbem
import re
import signal

from lmi.providers import ComputerSystem
from lmi.software.util.SoftwareConfiguration \
    import SoftwareConfiguration as Configuration

RE_INSTANCE_ID = re.compile(r'LMI:(?P<clsname>[a-z][a-z_0-9]+):(?P<id>\d+)',
    re.IGNORECASE)

RE_EVRA  = re.compile(
    r'^(?P<epoch>\d+):(?P<version>[~\w.+{}]+)-(?P<release>[\w.+{}]+)'
    r'\.(?P<arch>[^.]+)$')
RE_NEVRA = re.compile(
    r'^(?P<name>.+)-(?P<evra>(?P<epoch>\d+):(?P<version>[~\w.+{}]+)'
    r'-(?P<release>[\w.+{}]+)\.(?P<arch>[^.]+))$')
RE_NEVRA_OPT_EPOCH = re.compile(
    r'^(?P<name>.+)-(?P<evra>(?:(?P<epoch>\d+):)?(?P<version>[~\w.+{}]+)'
    r'-(?P<release>[\w.+{}]+)\.(?P<arch>[^.]+))$')
RE_ENVRA = re.compile(
    r'^(?P<epoch>\d+):(?P<name>.+)-(?P<evra>(?P<version>[~\w.+{}]+)'
    r'-(?P<release>[\w.+{}]+)\.(?P<arch>[^.]+))$')

def _get_distname():
    """
    @return name of linux distribution
    """
    if hasattr(platform, 'linux_distribution'):
        return platform.linux_distribution(
                full_distribution_name=False)[0].lower()
    else:
        return platform.dist()[0].lower()


def get_target_operating_system():
    """
    @return (val, text).
    Where val is a number from ValueMap of TargetOperatingSystem property
    of CIM_SoftwareElement class and text is its testual representation.
    """

    system = platform.system()
    if system.lower() == 'linux':
        try:
            val, dist = \
            { 'redhat'   : (79, 'RedHat Enterprise Linux')
            , 'suse'     : (81, 'SUSE')
            , 'mandriva' : (88, 'Mandriva')
            , 'ubuntu'   : (93, 'Ubuntu')
            , 'debian'   : (95, 'Debian')
            }[_get_distname()]
        except KeyError:
            linrel = platform.uname()[2]
            if linrel.startswith('2.4'):
                val, dist = (97, 'Linux 2.4.x')
            elif linrel.startswith('2.6'):
                val, dist = (99, 'Linux 2.6.x')
            else:
                return (36, 'LINUX') # no check for x86_64
        if platform.machine() == 'x86_64':
            val  += 1
            dist += ' 64-Bit'
        return (val, dist)
    elif system.lower() in ('macosx', 'darwin'):
        return (2, 'MACOS')
    # elif system.lower() == 'windows': # no general value
    else:
        return (0, 'Unknown')

def check_target_operating_system(system):
    """
    @return if param system matches current target operating system
    """
    if isinstance(system, basestring):
        system = int(system)
    if not isinstance(system, (int, long)):
        raise TypeError("system must be either string or integer, not %s" %
                system.__class__.__name__)
    tos = get_target_operating_system()
    if system == tos[0]:
        return True
    if system == 36: # linux
        if platform.system().lower() == "linux":
            return True
    if (   system >= 97 and system <= 100 # linux 2.x.x
       and platform.uname()[2].startswith('2.4' if system < 99 else '2.6')
       # check machine
       and (  bool(platform.machine().endswith('64'))
           == bool(not (system % 2)))):
        return True
    return False

def nevra2filter(nevra):
    """
    Takes either regexp match object resulting from RE_NEVRA match or
    a nevra string.
    @return dictionary with package filter key-value pairs made from nevra
    """
    if isinstance(nevra, basestring):
        match = RE_NEVRA_OPT_EPOCH.match(nevra)
    elif nevra.__class__.__name__.lower() == "sre_match":
        match = nevra
    else:
        raise TypeError("nevra must be either string or regexp match object")
    epoch = match.group("epoch")
    if not epoch or match.group("epoch").lower() == "(none)":
        epoch = "0"
    return { "name"    : match.group("name")
           , "epoch"   : epoch
           , "version" : match.group("version")
           , "release" : match.group("release")
           , "arch"    : match.group("arch")
           }

def make_nevra(name, epoch, version, release, arch, with_epoch='NOT_ZERO'):
    """
    @param with_epoch may be one of:
        "NOT_ZERO" - include epoch only if it's not zero
        "ALWAYS"   - include epoch always
        "NEVER"    - do not include epoch at all
    """
    estr = ''
    if with_epoch.lower() == "always":
        estr = epoch
    elif with_epoch.lower() == "not_zero":
        if epoch != "0":
            estr = epoch
    if len(estr):
        estr += ":"
    return "%s-%s%s-%s.%s" % (name, estr, version, release, arch)

def pkg2nevra(pkg, with_epoch='NOT_ZERO'):
    """
    @return nevra string made of pkg
    """
    return make_nevra(pkg.name, pkg.epoch, pkg.version,
            pkg.release, pkg.arch, with_epoch)

def get_signal_name(signal_num):
    """
    @return name of signal for signal_num argument
    """
    if not isinstance(signal_num, (int, long)):
        raise TypeError("signal_num must be an integer")
    try:
        return dict((v, k) for k, v in signal.__dict__.items())[signal_num]
    except KeyError:
        return "UNKNOWN_SIGNAL(%d)" % signal_num

def new_instance_name(class_name, namespace=None, **kwargs):
    """
    Create new CIMInstanceName with provided attributes.

    :param class_name: (``str``) Name of CIM Class that will be instantiated.
    :param namespace: (``str``) Target CIM namespace.
    :param kwargs: (``dict``) Key properties with associated values.
    :rtype: (``pywbem.CIMInstanceName``) New object path.
    """
    keybindings = pywbem.NocaseDict(kwargs)
    if namespace is None:
        namespace = Configuration.get_instance().namespace
    try:
        host = ComputerSystem.get_system_name()
    except ComputerSystem.UninitializedError:
        host = None
    return pywbem.CIMInstanceName(
            class_name,
            host=host,
            namespace=namespace,
            keybindings=keybindings)



# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#
"""
Base class and utilities for all OpenLMI Account tests.
"""

import os

from lmi.test import cimbase

class AccountBase(cimbase.CIMTestCase):
    """
    Base class for all LMI Account tests.
    """

    @classmethod
    def setUpClass(cls):
        cimbase.CIMTestCase.setUpClass.im_func(cls)
        cls.user_name = os.environ.get("LMI_ACCOUNT_USER")
        cls.group_name = os.environ.get("LMI_ACCOUNT_GROUP")




# OpenLMI tests
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
#

from lmi.test.lmibase import LmiTestCase
import lmi.shell
import unittest
import pywbem
import traceback
import sys

class TestInstances(LmiTestCase):
    """
    Test that EnumerateInstances, EnumerateInstanceNames, GetInstances,
    Associators and References works on all LMI_* classes.

    It does not check actual content of the instances, just that the above
    methods do not crash.
    """
    SKIP_CLASSES = [
            # Enumeration is not implemented (and never will be):
            "LMI_SoftwareIdentityFileCheck",
            "LMI_FIFOPipeFile",
            "LMI_DataFile",
            "LMI_FileIdentity",
            "LMI_UnixDirectory",
            "LMI_UnixFile",
            "LMI_UnixDeviceFile",
            "LMI_DirectoryContainsFile",
            "LMI_SymbolicLink",
            "LMI_UnixSocket",
            "LMI_SoftwareInstModification",
            "LMI_StorageInstModification",
            "LMI_SoftwareIdentity",
            "LMI_SoftwareIdentityChecks",
            "LMI_SoftwareInstCreation",
            "LMI_StorageInstCreation",
            "LMI_AccountInstanceDeletionIndication",
            "LMI_NetworkInstCreation",
            "LMI_NetworkInstModification",
            "LMI_NetworkInstDeletion",
            "LMI_JournalLogRecordInstanceCreationIndication",
            "LMI_ServiceInstanceModificationIndication",
            "LMI_AccountInstanceCreationIndication",

            # too much memory needed, TODO?
            "LMI_ResourceForSoftwareIdentity",
            "LMI_MemberOfSoftwareCollection",
            "LMI_SoftwareInstallationServiceAffectsElement",
    ]

    SKIP_ASSOCIATORS = [
            # too many associators(), too much memory needed:
            "LMI_SoftwareIdentityResource",
            "LMI_InstalledSoftwareIdentity",
            "LMI_SoftwareInstallationService",
            "LMI_SystemSoftwareCollection",
    ]

    def compare_paths(self, instance_names, instances):
        """
        Compare list of LMIInstanceNames and LMIInstances.
        Raise assertion error if these two lists do not represent the same
        CIM instances.
        """
        # Check every instance has appropriate instance_name
        for i in instances:
            path = i.path
            self.assertCIMNameIn(path, instance_names)

        # The other way around - check every instance_name has appropriate
        # instance
        instances_paths = [i.path for i in instances]
        for i in instance_names:
            self.assertCIMNameIn(i, instances_paths)

        # Check that there are no duplicates in instance_names
        # (this automatically ensures there are no duplicates in instances)
        inames_set = set(instance_names)
        self.assertEqual(len(inames_set), len(instance_names))

        # check, that instance key properties match their counterparts in path
        for i in instances:
            for key in i.path.key_properties():
                self.assertEqual(getattr(i, key), getattr(i.path, key))

    def check_instances(self, cls, skip_associators):
        """
        Check that EnumerateInstances and EnumerateInstanceNames work and
        return the same objects.
        """
        instances = cls.instances()
        instance_names = cls.instance_names()

        # Check that EnumerateInstances and EnumerateInstanceNames returns the
        # same instances
        self.compare_paths(instance_names, instances)

        # try GetInstance on each name, just to check it works
        for i in instance_names:
            i.to_instance()

        if skip_associators:
            return

        element_class = self.conn.root.cimv2.CIM_ManagedElement
        for instance in instances:
            if not lmi.shell.LMIUtil.lmi_isinstance(instance, element_class):
                # skip this class, it cannot have associations
                return
            # call Associators & AssociatorNames
            associators = instance.associators()
            associator_names = instance.associator_names()

            # Check that Associators and AssociatorNames returns the
            # same instances
            self.compare_paths(associator_names, associators)

            # call References & ReferenceNames
            references = instance.references()
            reference_names = instance.reference_names()

            # Check that References and ReferenceNames returns the
            # same instances
            self.compare_paths(reference_names, references)


    def test_all(self):
        """
        Enumerate all classes and start tests for all LMI_ ones.
        """
        lmi.shell.LMIUtil.lmi_set_use_exceptions(True)
        classes = self.conn.root.cimv2.classes()
        # do not stop on the first exception, scan all classes
        exception_count = 0

        for name in classes:
            try:
                if not name.startswith("LMI_"):
                    continue
                if name in self.SKIP_CLASSES:
                    print >>sys.stderr, "Skipping", name
                    continue
                skip_associators = name in self.SKIP_ASSOCIATORS
                print >>sys.stderr, "Checking ", name
                cls = lmi.shell.LMIClass(self.conn, self.conn.root.cimv2, name)
                self.check_instances(cls, skip_associators)
            except Exception, exc:
                print >>sys.stderr, "ERROR scanning class", cls, exc
                traceback.print_exc()
                exception_count += 1

        self.assertEqual(exception_count, 0)

if __name__ == '__main__':
    # Run the standard unittest.main() function to load and run all tests
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Simple parser of MOF files. The only thing that is parsed out is list of
included files. This list is composed recursively, i.e. a MOF file can
include another MOF file, which can then include another one.
"""
# Based on pywbem.mof_compiler.py, (C) Copyright 2006-2007 Novell, Inc.

from pywbem import lex
from pywbem import yacc
from pywbem.lex import TOKEN
import os

class MOFLexer:
    """ Lexer for MOF files. """
    def __init__(self, **kwargs):
        self.lexer = lex.lex(module=self, **kwargs)

    reserved = {
        'pragma':'PRAGMA',
        'false': 'FALSE',
        'true': 'TRUE',
        'null': 'NULL'
        }

    tokens = reserved.values() + [
            'IDENTIFIER',
            'stringValue',
            'floatValue',
            'charValue',
            'binaryValue',
            'octalValue',
            'decimalValue',
            'hexValue',
        ]

    literals = '#(){};[],$:='

    # UTF-8 (from Unicode 4.0.0 standard):
    # Table 3-6. Well-Formed UTF-8 Byte Sequences Code Points
    # 1st Byte 2nd Byte 3rd Byte 4th Byte
    # U+0000..U+007F     00..7F
    # U+0080..U+07FF     C2..DF   80..BF
    # U+0800..U+0FFF     E0       A0..BF   80..BF
    # U+1000..U+CFFF     E1..EC   80..BF   80..BF
    # U+D000..U+D7FF     ED       80..9F   80..BF
    # U+E000..U+FFFF     EE..EF   80..BF   80..BF
    # U+10000..U+3FFFF   F0       90..BF   80..BF   80..BF
    # U+40000..U+FFFFF   F1..F3   80..BF   80..BF   80..BF
    # U+100000..U+10FFFF F4       80..8F   80..BF   80..BF

    utf8_2 = r'[\xC2-\xDF][\x80-\xBF]'
    utf8_3_1 = r'\xE0[\xA0-\xBF][\x80-\xBF]'
    utf8_3_2 = r'[\xE1-\xEC][\x80-\xBF][\x80-\xBF]'
    utf8_3_3 = r'\xED[\x80-\x9F][\x80-\xBF]'
    utf8_3_4 = r'[\xEE-\xEF][\x80-\xBF][\x80-\xBF]'
    utf8_4_1 = r'\xF0[\x90-\xBF][\x80-\xBF][\x80-\xBF]'
    utf8_4_2 = r'[\xF1-\xF3][\x80-\xBF][\x80-\xBF][\x80-\xBF]'
    utf8_4_3 = r'\xF4[\x80-\x8F][\x80-\xBF][\x80-\xBF]'

    utf8Char = r'(%s)|(%s)|(%s)|(%s)|(%s)|(%s)|(%s)|(%s)' % (utf8_2, utf8_3_1,
            utf8_3_2, utf8_3_3, utf8_3_4, utf8_4_1, utf8_4_2, utf8_4_3)

    def t_COMMENT(self, t):
        r'//.*'
        pass

    def t_MCOMMENT(self, t):
        r'/\*(.|\n)*?\*/'
        t.lineno += t.value.count('\n')


    t_binaryValue = r'[+-]?[01]+[bB]'
    t_octalValue = r'[+-]?0[0-7]+'
    t_decimalValue = r'[+-]?([1-9][0-9]*|0)'
    t_hexValue = r'[+-]?0[xX][0-9a-fA-F]+'
    t_floatValue = r'[+-]?[0-9]*\.[0-9]+([eE][+-]?[0-9]+)?'

    simpleEscape = r"""[bfnrt'"\\]"""
    hexEscape = r'x[0-9a-fA-F]{1,4}'
    escapeSequence = r'[\\]((%s)|(%s))' % (simpleEscape, hexEscape)
    cChar = r"[^'\\\n\r]|(%s)" % escapeSequence
    sChar = r'[^"\\\n\r]|(%s)' % escapeSequence
    charValue = r"'%s'" % cChar

    t_stringValue = r'"(%s)*"' % sChar

    identifier_re = r'([a-zA-Z_]|(%s))([0-9a-zA-Z_]|(%s))*' % (utf8Char, utf8Char)

    @TOKEN(identifier_re)
    def t_IDENTIFIER(self, t):
        t.type = self.reserved.get(t.value.lower(), 'IDENTIFIER')  # check for reserved word
        return t

    # Define a rule so we can track line numbers
    def t_newline(self, t):
        r'\n+'
        t.lexer.lineno += len(t.value)
        t.lexer.linestart = t.lexpos

    t_ignore = ' \r\t'

    # Error handling rule
    def t_error(self, t):
        msg = "Illegal character '%s' " % t.value[0]
        msg += "Line %d" % (t.lineno)
        t.lexer.parser.log(msg)
        t.lexer.skip(1)

class MOFParseError(ValueError):
    pass

class MOFParser:
    """ Parser of '#pragma include' directivers in MOF files."""
    tokens = MOFLexer.tokens

    def __init__(self, **kwargs):
        self.lexer = MOFLexer()
        self.parser = yacc.yacc(module=self, **kwargs)
        self.files = []

    def p_error(self, p):
        ex = MOFParseError('Parse error at line %d' % (p.lineno))
        if p is None:
            ex.args = ('Unexpected end of file',)
            raise ex
        ex.file = self.filename
        ex.lineno = p.lineno
        raise ex


    def p_mofSpecification(self, p):
        """mof : mofItemList"""

    def p_mofItemList(self, p):
        """mofItemList : empty
                         | mofItemList mofItem
                       """

    def p_mofItem(self, p):
        """mofItem : pragma
                     | IDENTIFIER
                     | literal
                     | value
                     """


    def p_pragma(self, p):
        """pragma : '#' PRAGMA pragmaName '(' pragmaParameter ')'"""
        directive = p[3].lower()
        param = p[5]
        if directive == 'include':
            fname = param
            fname = os.path.dirname(self.filename) + '/' + fname
            self._parse_file(fname)

    def p_pragmaName(self, p):
        """pragmaName : identifier"""
        p[0] = p[1]

    def p_pragmaParameter(self, p):
        """pragmaParameter : stringValue"""
        p[0] = self._fixStringValue(p[1])

    def _fixStringValue(self, s):
        s = s[1:-1]
        rv = ''
        esc = False
        i = -1
        while i < len(s) - 1:
            i += 1
            ch = s[i]
            if ch == '\\' and not esc:
                esc = True
                continue
            if not esc:
                rv += ch
                continue

            if ch == '"'   : rv += '"'
            elif ch == 'n' : rv += '\n'
            elif ch == 't' : rv += '\t'
            elif ch == 'b' : rv += '\b'
            elif ch == 'f' : rv += '\f'
            elif ch == 'r' : rv += '\r'
            elif ch == '\\': rv += '\\'
            elif ch in ['x', 'X']:
                hexc = 0
                j = 0
                i += 1
                while j < 4:
                    c = s[i + j];
                    c = c.upper()
                    if not c.isdigit() and not c in 'ABCDEF':
                        break;
                    hexc <<= 4
                    if c.isdigit():
                        hexc |= ord(c) - ord('0')
                    else:
                        hexc |= ord(c) - ord('A') + 0XA
                    j += 1
                rv += chr(hexc)
                i += j - 1

            esc = False

        return rv

    def p_value(self, p):
        """value : integerValue
                 | floatValue
                 | charValue
                 | stringValue
                 | booleanValue
                 | nullValue
                 """

    def p_literal(self, p):
        """literal : '('
                  | ')'
                  | '{'
                  | '}'
                  | ';'
                  | '['
                  | ']'
                  | ','
                  | '$'
                  | ':'
                  | '='
                  """

    def p_integerValue(self, p):
        """integerValue : binaryValue
                        | octalValue
                        | decimalValue
                        | hexValue
                        """

    def p_booleanValue(self, p):
        """booleanValue : FALSE
                        | TRUE
                        """
    def p_nullValue(self, p):
        """nullValue : NULL"""

    def p_identifier(self, p):
        """identifier : IDENTIFIER
                      """
        p[0] = p[1]

    def p_empty(self, p):
        'empty :'


    def _parse_file(self, fname):
        f = open(fname, 'r')
        mof = f.read()
        f.close()

        self.files.append(fname)

        old_filename = self.filename
        self.filename = fname

        # we must use fresh lexer so the old one can continue with parsing the
        # old file
        lex = self.lexer.lexer.clone()
        lex.parser = self.parser
        self.parser.parse(mof, lexer=lex)
        self.filename = old_filename


    def parse_includes(self, fnames):
        """
        Parse given MOF files and return array with all parsed files,
        including the included ones.
        """
        self.filename = '__main__'
        self.files = []

        for fname in fnames:
            if fname[0] != '/':
                fname = os.path.curdir + '/' + fname
            self._parse_file(fname)

        return self.files



#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_SystemSoftwareCollection`` provider.
"""

import unittest

import swbase

class TestSystemSoftwareCollection(swbase.SwTestCase):
    """
    Basic cim operations test on ``LMI_SystemSoftwareCollection``.
    """

    CLASS_NAME = "LMI_SystemSoftwareCollection"
    KEYS = ("InstanceID", )

    def make_op(self):
        """
        :returns Object path of ``LMI_SystemSoftwareCollection``.
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return self.cim_class.new_instance_name({
            "InstanceID" : "LMI:LMI_SystemSoftwareCollection"
        })

    def test_get_instance(self):
        """
        Test ``GetInstance()`` call on ``LMI_SystemSoftwareCollection``.
        """
        objpath = self.make_op()
        inst = objpath.to_instance()

        self.assertCIMNameEqual(objpath, inst.path)
        for key in self.KEYS:
            self.assertEqual(getattr(inst, key), getattr(objpath, key))
        self.assertIsInstance(inst.Caption, basestring)

    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames()`` call on ``LMI_SystemSoftwareCollection``.
        """
        inames = self.cim_class.instance_names()
        self.assertEqual(len(inames), 1)
        iname = inames[0]
        self.assertCIMNameEqual(iname, self.make_op())

    def test_enum_instances(self):
        """
        Test ``EnumInstances()`` call on ``LMI_SystemSoftwareCollection``.
        """
        insts = self.cim_class.instances()
        self.assertEqual(len(insts), 1)
        inst = insts[0]
        self.assertCIMNameEqual(inst.path, self.make_op())
        self.assertEqual(inst.InstanceID, inst.InstanceID)
        self.assertIsInstance(inst.Caption, basestring)

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestSystemSoftwareCollection)

if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Just a common functionality related to class CIM_Error.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.software import util

class Values(object):
    class ErrorSourceFormat(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        CIMObjectPath = pywbem.Uint16(2)
        # DMTF_Reserved = ..
        _reverse_map = {
                0 : 'Unknown',
                1 : 'Other',
                2 : 'CIMObjectPath'
        }

    class ErrorType(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Communications_Error = pywbem.Uint16(2)
        Quality_of_Service_Error = pywbem.Uint16(3)
        Software_Error = pywbem.Uint16(4)
        Hardware_Error = pywbem.Uint16(5)
        Environmental_Error = pywbem.Uint16(6)
        Security_Error = pywbem.Uint16(7)
        Oversubscription_Error = pywbem.Uint16(8)
        Unavailable_Resource_Error = pywbem.Uint16(9)
        Unsupported_Operation_Error = pywbem.Uint16(10)
        # DMTF_Reserved = ..
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'Communications Error',
                3  : 'Quality of Service Error',
                4  : 'Software Error',
                5  : 'Hardware Error',
                6  : 'Environmental Error',
                7  : 'Security Error',
                8  : 'Oversubscription Error',
                9  : 'Unavailable Resource Error',
                10 : 'Unsupported Operation Error'
        }

    class CIMStatusCode(object):
        CIM_ERR_FAILED = pywbem.Uint32(1)
        CIM_ERR_ACCESS_DENIED = pywbem.Uint32(2)
        CIM_ERR_INVALID_NAMESPACE = pywbem.Uint32(3)
        CIM_ERR_INVALID_PARAMETER = pywbem.Uint32(4)
        CIM_ERR_INVALID_CLASS = pywbem.Uint32(5)
        CIM_ERR_NOT_FOUND = pywbem.Uint32(6)
        CIM_ERR_NOT_SUPPORTED = pywbem.Uint32(7)
        CIM_ERR_CLASS_HAS_CHILDREN = pywbem.Uint32(8)
        CIM_ERR_CLASS_HAS_INSTANCES = pywbem.Uint32(9)
        CIM_ERR_INVALID_SUPERCLASS = pywbem.Uint32(10)
        CIM_ERR_ALREADY_EXISTS = pywbem.Uint32(11)
        CIM_ERR_NO_SUCH_PROPERTY = pywbem.Uint32(12)
        CIM_ERR_TYPE_MISMATCH = pywbem.Uint32(13)
        CIM_ERR_QUERY_LANGUAGE_NOT_SUPPORTED = pywbem.Uint32(14)
        CIM_ERR_INVALID_QUERY = pywbem.Uint32(15)
        CIM_ERR_METHOD_NOT_AVAILABLE = pywbem.Uint32(16)
        CIM_ERR_METHOD_NOT_FOUND = pywbem.Uint32(17)
        CIM_ERR_UNEXPECTED_RESPONSE = pywbem.Uint32(18)
        CIM_ERR_INVALID_RESPONSE_DESTINATION = pywbem.Uint32(19)
        CIM_ERR_NAMESPACE_NOT_EMPTY = pywbem.Uint32(20)
        CIM_ERR_INVALID_ENUMERATION_CONTEXT = pywbem.Uint32(21)
        CIM_ERR_INVALID_OPERATION_TIMEOUT = pywbem.Uint32(22)
        CIM_ERR_PULL_HAS_BEEN_ABANDONED = pywbem.Uint32(23)
        CIM_ERR_PULL_CANNOT_BE_ABANDONED = pywbem.Uint32(24)
        CIM_ERR_FILTERED_ENUMERATION_NOT_SUPPORTED = pywbem.Uint32(25)
        CIM_ERR_CONTINUATION_ON_ERROR_NOT_SUPPORTED = pywbem.Uint32(26)
        CIM_ERR_SERVER_LIMITS_EXCEEDED = pywbem.Uint32(27)
        CIM_ERR_SERVER_IS_SHUTTING_DOWN = pywbem.Uint32(28)
        CIM_ERR_QUERY_FEATURE_NOT_SUPPORTED = pywbem.Uint32(29)
        # DMTF_Reserved = ..
        _reverse_map = {
                1  : 'CIM_ERR_FAILED',
                2  : 'CIM_ERR_ACCESS_DENIED',
                3  : 'CIM_ERR_INVALID_NAMESPACE',
                4  : 'CIM_ERR_INVALID_PARAMETER',
                5  : 'CIM_ERR_INVALID_CLASS',
                6  : 'CIM_ERR_NOT_FOUND',
                7  : 'CIM_ERR_NOT_SUPPORTED',
                8  : 'CIM_ERR_CLASS_HAS_CHILDREN',
                9  : 'CIM_ERR_CLASS_HAS_INSTANCES',
                10 : 'CIM_ERR_INVALID_SUPERCLASS',
                11 : 'CIM_ERR_ALREADY_EXISTS',
                12 : 'CIM_ERR_NO_SUCH_PROPERTY',
                13 : 'CIM_ERR_TYPE_MISMATCH',
                14 : 'CIM_ERR_QUERY_LANGUAGE_NOT_SUPPORTED',
                15 : 'CIM_ERR_INVALID_QUERY',
                16 : 'CIM_ERR_METHOD_NOT_AVAILABLE',
                17 : 'CIM_ERR_METHOD_NOT_FOUND',
                18 : 'CIM_ERR_UNEXPECTED_RESPONSE',
                19 : 'CIM_ERR_INVALID_RESPONSE_DESTINATION',
                20 : 'CIM_ERR_NAMESPACE_NOT_EMPTY',
                21 : 'CIM_ERR_INVALID_ENUMERATION_CONTEXT',
                22 : 'CIM_ERR_INVALID_OPERATION_TIMEOUT',
                23 : 'CIM_ERR_PULL_HAS_BEEN_ABANDONED',
                24 : 'CIM_ERR_PULL_CANNOT_BE_ABANDONED',
                25 : 'CIM_ERR_FILTERED_ENUMERATION_NOT_SUPPORTED',
                26 : 'CIM_ERR_CONTINUATION_ON_ERROR_NOT_SUPPORTED',
                27 : 'CIM_ERR_SERVER_LIMITS_EXCEEDED',
                28 : 'CIM_ERR_SERVER_IS_SHUTTING_DOWN',
                29 : 'CIM_ERR_QUERY_FEATURE_NOT_SUPPORTED'
        }

    class PerceivedSeverity(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Information = pywbem.Uint16(2)
        Degraded_Warning = pywbem.Uint16(3)
        Minor = pywbem.Uint16(4)
        Major = pywbem.Uint16(5)
        Critical = pywbem.Uint16(6)
        Fatal_NonRecoverable = pywbem.Uint16(7)
        # DMTF_Reserved = ..
        _reverse_map = {
                0 : 'Unknown',
                1 : 'Other',
                2 : 'Information',
                3 : 'Degraded/Warning',
                4 : 'Minor',
                5 : 'Major',
                6 : 'Critical',
                7 : 'Fatal/NonRecoverable'
        }

    class ProbableCause(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Adapter_Card_Error = pywbem.Uint16(2)
        Application_Subsystem_Failure = pywbem.Uint16(3)
        Bandwidth_Reduced = pywbem.Uint16(4)
        Connection_Establishment_Error = pywbem.Uint16(5)
        Communications_Protocol_Error = pywbem.Uint16(6)
        Communications_Subsystem_Failure = pywbem.Uint16(7)
        Configuration_Customization_Error = pywbem.Uint16(8)
        Congestion = pywbem.Uint16(9)
        Corrupt_Data = pywbem.Uint16(10)
        CPU_Cycles_Limit_Exceeded = pywbem.Uint16(11)
        Dataset_Modem_Error = pywbem.Uint16(12)
        Degraded_Signal = pywbem.Uint16(13)
        DTE_DCE_Interface_Error = pywbem.Uint16(14)
        Enclosure_Door_Open = pywbem.Uint16(15)
        Equipment_Malfunction = pywbem.Uint16(16)
        Excessive_Vibration = pywbem.Uint16(17)
        File_Format_Error = pywbem.Uint16(18)
        Fire_Detected = pywbem.Uint16(19)
        Flood_Detected = pywbem.Uint16(20)
        Framing_Error = pywbem.Uint16(21)
        HVAC_Problem = pywbem.Uint16(22)
        Humidity_Unacceptable = pywbem.Uint16(23)
        I_O_Device_Error = pywbem.Uint16(24)
        Input_Device_Error = pywbem.Uint16(25)
        LAN_Error = pywbem.Uint16(26)
        Non_Toxic_Leak_Detected = pywbem.Uint16(27)
        Local_Node_Transmission_Error = pywbem.Uint16(28)
        Loss_of_Frame = pywbem.Uint16(29)
        Loss_of_Signal = pywbem.Uint16(30)
        Material_Supply_Exhausted = pywbem.Uint16(31)
        Multiplexer_Problem = pywbem.Uint16(32)
        Out_of_Memory = pywbem.Uint16(33)
        Output_Device_Error = pywbem.Uint16(34)
        Performance_Degraded = pywbem.Uint16(35)
        Power_Problem = pywbem.Uint16(36)
        Pressure_Unacceptable = pywbem.Uint16(37)
        Processor_Problem__Internal_Machine_Error_ = pywbem.Uint16(38)
        Pump_Failure = pywbem.Uint16(39)
        Queue_Size_Exceeded = pywbem.Uint16(40)
        Receive_Failure = pywbem.Uint16(41)
        Receiver_Failure = pywbem.Uint16(42)
        Remote_Node_Transmission_Error = pywbem.Uint16(43)
        Resource_at_or_Nearing_Capacity = pywbem.Uint16(44)
        Response_Time_Excessive = pywbem.Uint16(45)
        Retransmission_Rate_Excessive = pywbem.Uint16(46)
        Software_Error = pywbem.Uint16(47)
        Software_Program_Abnormally_Terminated = pywbem.Uint16(48)
        Software_Program_Error__Incorrect_Results_ = pywbem.Uint16(49)
        Storage_Capacity_Problem = pywbem.Uint16(50)
        Temperature_Unacceptable = pywbem.Uint16(51)
        Threshold_Crossed = pywbem.Uint16(52)
        Timing_Problem = pywbem.Uint16(53)
        Toxic_Leak_Detected = pywbem.Uint16(54)
        Transmit_Failure = pywbem.Uint16(55)
        Transmitter_Failure = pywbem.Uint16(56)
        Underlying_Resource_Unavailable = pywbem.Uint16(57)
        Version_Mismatch = pywbem.Uint16(58)
        Previous_Alert_Cleared = pywbem.Uint16(59)
        Login_Attempts_Failed = pywbem.Uint16(60)
        Software_Virus_Detected = pywbem.Uint16(61)
        Hardware_Security_Breached = pywbem.Uint16(62)
        Denial_of_Service_Detected = pywbem.Uint16(63)
        Security_Credential_Mismatch = pywbem.Uint16(64)
        Unauthorized_Access = pywbem.Uint16(65)
        Alarm_Received = pywbem.Uint16(66)
        Loss_of_Pointer = pywbem.Uint16(67)
        Payload_Mismatch = pywbem.Uint16(68)
        Transmission_Error = pywbem.Uint16(69)
        Excessive_Error_Rate = pywbem.Uint16(70)
        Trace_Problem = pywbem.Uint16(71)
        Element_Unavailable = pywbem.Uint16(72)
        Element_Missing = pywbem.Uint16(73)
        Loss_of_Multi_Frame = pywbem.Uint16(74)
        Broadcast_Channel_Failure = pywbem.Uint16(75)
        Invalid_Message_Received = pywbem.Uint16(76)
        Routing_Failure = pywbem.Uint16(77)
        Backplane_Failure = pywbem.Uint16(78)
        Identifier_Duplication = pywbem.Uint16(79)
        Protection_Path_Failure = pywbem.Uint16(80)
        Sync_Loss_or_Mismatch = pywbem.Uint16(81)
        Terminal_Problem = pywbem.Uint16(82)
        Real_Time_Clock_Failure = pywbem.Uint16(83)
        Antenna_Failure = pywbem.Uint16(84)
        Battery_Charging_Failure = pywbem.Uint16(85)
        Disk_Failure = pywbem.Uint16(86)
        Frequency_Hopping_Failure = pywbem.Uint16(87)
        Loss_of_Redundancy = pywbem.Uint16(88)
        Power_Supply_Failure = pywbem.Uint16(89)
        Signal_Quality_Problem = pywbem.Uint16(90)
        Battery_Discharging = pywbem.Uint16(91)
        Battery_Failure = pywbem.Uint16(92)
        Commercial_Power_Problem = pywbem.Uint16(93)
        Fan_Failure = pywbem.Uint16(94)
        Engine_Failure = pywbem.Uint16(95)
        Sensor_Failure = pywbem.Uint16(96)
        Fuse_Failure = pywbem.Uint16(97)
        Generator_Failure = pywbem.Uint16(98)
        Low_Battery = pywbem.Uint16(99)
        Low_Fuel = pywbem.Uint16(100)
        Low_Water = pywbem.Uint16(101)
        Explosive_Gas = pywbem.Uint16(102)
        High_Winds = pywbem.Uint16(103)
        Ice_Buildup = pywbem.Uint16(104)
        Smoke = pywbem.Uint16(105)
        Memory_Mismatch = pywbem.Uint16(106)
        Out_of_CPU_Cycles = pywbem.Uint16(107)
        Software_Environment_Problem = pywbem.Uint16(108)
        Software_Download_Failure = pywbem.Uint16(109)
        Element_Reinitialized = pywbem.Uint16(110)
        Timeout = pywbem.Uint16(111)
        Logging_Problems = pywbem.Uint16(112)
        Leak_Detected = pywbem.Uint16(113)
        Protection_Mechanism_Failure = pywbem.Uint16(114)
        Protecting_Resource_Failure = pywbem.Uint16(115)
        Database_Inconsistency = pywbem.Uint16(116)
        Authentication_Failure = pywbem.Uint16(117)
        Breach_of_Confidentiality = pywbem.Uint16(118)
        Cable_Tamper = pywbem.Uint16(119)
        Delayed_Information = pywbem.Uint16(120)
        Duplicate_Information = pywbem.Uint16(121)
        Information_Missing = pywbem.Uint16(122)
        Information_Modification = pywbem.Uint16(123)
        Information_Out_of_Sequence = pywbem.Uint16(124)
        Key_Expired = pywbem.Uint16(125)
        Non_Repudiation_Failure = pywbem.Uint16(126)
        Out_of_Hours_Activity = pywbem.Uint16(127)
        Out_of_Service = pywbem.Uint16(128)
        Procedural_Error = pywbem.Uint16(129)
        Unexpected_Information = pywbem.Uint16(130)
        # DMTF_Reserved = ..
        _reverse_map = {
                0   : 'Unknown',
                1   : 'Other',
                2   : 'Adapter/Card Error',
                3   : 'Application Subsystem Failure',
                4   : 'Bandwidth Reduced',
                5   : 'Connection Establishment Error',
                6   : 'Communications Protocol Error',
                7   : 'Communications Subsystem Failure',
                8   : 'Configuration/Customization Error',
                9   : 'Congestion',
                10  : 'Corrupt Data',
                11  : 'CPU Cycles Limit Exceeded',
                12  : 'Dataset/Modem Error',
                13  : 'Degraded Signal',
                14  : 'DTE-DCE Interface Error',
                15  : 'Enclosure Door Open',
                16  : 'Equipment Malfunction',
                17  : 'Excessive Vibration',
                18  : 'File Format Error',
                19  : 'Fire Detected',
                20  : 'Flood Detected',
                21  : 'Framing Error',
                22  : 'HVAC Problem',
                23  : 'Humidity Unacceptable',
                24  : 'I/O Device Error',
                25  : 'Input Device Error',
                26  : 'LAN Error',
                27  : 'Non-Toxic Leak Detected',
                28  : 'Local Node Transmission Error',
                29  : 'Loss of Frame',
                30  : 'Loss of Signal',
                31  : 'Material Supply Exhausted',
                32  : 'Multiplexer Problem',
                33  : 'Out of Memory',
                34  : 'Output Device Error',
                35  : 'Performance Degraded',
                36  : 'Power Problem',
                37  : 'Pressure Unacceptable',
                38  : 'Processor Problem (Internal Machine Error)',
                39  : 'Pump Failure',
                40  : 'Queue Size Exceeded',
                41  : 'Receive Failure',
                42  : 'Receiver Failure',
                43  : 'Remote Node Transmission Error',
                44  : 'Resource at or Nearing Capacity',
                45  : 'Response Time Excessive',
                46  : 'Retransmission Rate Excessive',
                47  : 'Software Error',
                48  : 'Software Program Abnormally Terminated',
                49  : 'Software Program Error (Incorrect Results)',
                50  : 'Storage Capacity Problem',
                51  : 'Temperature Unacceptable',
                52  : 'Threshold Crossed',
                53  : 'Timing Problem',
                54  : 'Toxic Leak Detected',
                55  : 'Transmit Failure',
                56  : 'Transmitter Failure',
                57  : 'Underlying Resource Unavailable',
                58  : 'Version Mismatch',
                59  : 'Previous Alert Cleared',
                60  : 'Login Attempts Failed',
                61  : 'Software Virus Detected',
                62  : 'Hardware Security Breached',
                63  : 'Denial of Service Detected',
                64  : 'Security Credential Mismatch',
                65  : 'Unauthorized Access',
                66  : 'Alarm Received',
                67  : 'Loss of Pointer',
                68  : 'Payload Mismatch',
                69  : 'Transmission Error',
                70  : 'Excessive Error Rate',
                71  : 'Trace Problem',
                72  : 'Element Unavailable',
                73  : 'Element Missing',
                74  : 'Loss of Multi Frame',
                75  : 'Broadcast Channel Failure',
                76  : 'Invalid Message Received',
                77  : 'Routing Failure',
                78  : 'Backplane Failure',
                79  : 'Identifier Duplication',
                80  : 'Protection Path Failure',
                81  : 'Sync Loss or Mismatch',
                82  : 'Terminal Problem',
                83  : 'Real Time Clock Failure',
                84  : 'Antenna Failure',
                85  : 'Battery Charging Failure',
                86  : 'Disk Failure',
                87  : 'Frequency Hopping Failure',
                88  : 'Loss of Redundancy',
                89  : 'Power Supply Failure',
                90  : 'Signal Quality Problem',
                91  : 'Battery Discharging',
                92  : 'Battery Failure',
                93  : 'Commercial Power Problem',
                94  : 'Fan Failure',
                95  : 'Engine Failure',
                96  : 'Sensor Failure',
                97  : 'Fuse Failure',
                98  : 'Generator Failure',
                99  : 'Low Battery',
                100 : 'Low Fuel',
                101 : 'Low Water',
                102 : 'Explosive Gas',
                103 : 'High Winds',
                104 : 'Ice Buildup',
                105 : 'Smoke',
                106 : 'Memory Mismatch',
                107 : 'Out of CPU Cycles',
                108 : 'Software Environment Problem',
                109 : 'Software Download Failure',
                110 : 'Element Reinitialized',
                111 : 'Timeout',
                112 : 'Logging Problems',
                113 : 'Leak Detected',
                114 : 'Protection Mechanism Failure',
                115 : 'Protecting Resource Failure',
                116 : 'Database Inconsistency',
                117 : 'Authentication Failure',
                118 : 'Breach of Confidentiality',
                119 : 'Cable Tamper',
                120 : 'Delayed Information',
                121 : 'Duplicate Information',
                122 : 'Information Missing',
                123 : 'Information Modification',
                124 : 'Information Out of Sequence',
                125 : 'Key Expired',
                126 : 'Non-Repudiation Failure',
                127 : 'Out of Hours Activity',
                128 : 'Out of Service',
                129 : 'Procedural Error',
                130 : 'Unexpected Information'
        }

@cmpi_logging.trace_function
def make_instance(
        env,
        status_code=Values.CIMStatusCode.CIM_ERR_FAILED,
        error_type=Values.ErrorType.Software_Error,
        probable_cause=Values.ErrorType.Unknown,
        error_source=None,
        status_code_description=None,
        message=None,
        message_arguments=None,
        probable_cause_description=None,
        recommended_actions=None):
    for param in ('status_code', 'probable_cause', 'error_type'):
        if not isinstance(locals()[param], (int, long)):
            raise TypeError('%s must be integer'%param)
    if error_source is None:
        # this is a cyclic dependency
        from lmi.software.core import InstallationService
        error_source = InstallationService.get_path(env)
    if not isinstance(error_source, pywbem.CIMInstanceName):
        raise TypeError('error_source must be a CIMInstanceName')

    inst = pywbem.CIMInstance(classname="CIM_Error",
            path=util.new_instance_name("CIM_Error"))
    inst['CIMStatusCode'] = pywbem.Uint32(status_code)
    if status_code_description is not None:
        inst['CIMStatusCodeDescription'] = status_code_description
    inst['ErrorType'] = pywbem.Uint16(error_type)
    inst['ErrorSource'] = str(error_source)
    inst['ErrorSourceFormat'] = Values.ErrorSourceFormat.CIMObjectPath
    if message is not None:
        inst['Message'] = message
    if message_arguments is not None:
        inst['MessageArguments'] = message_arguments
    inst['ProbableCause'] = pywbem.Uint16(probable_cause)
    if probable_cause_description is not None:
        inst['ProbableCauseDescription'] = probable_cause_description
    if recommended_actions is not None:
        inst['RecommendedActions'] = recommended_actions
    return inst




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Just a common functionality related to LMI_SoftwareIdentity provider.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.software import util
from lmi.software.yumdb import PackageInfo, YumDB

LOG = cmpi_logging.get_logger(__name__)

class Values(object):
    class DetailedStatus(object):
        Not_Available = pywbem.Uint16(0)
        No_Additional_Information = pywbem.Uint16(1)
        Stressed = pywbem.Uint16(2)
        Predictive_Failure = pywbem.Uint16(3)
        Non_Recoverable_Error = pywbem.Uint16(4)
        Supporting_Entity_in_Error = pywbem.Uint16(5)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..

    class Status(object):
        OK = 'OK'
        Error = 'Error'
        Degraded = 'Degraded'
        Unknown = 'Unknown'
        Pred_Fail = 'Pred Fail'
        Starting = 'Starting'
        Stopping = 'Stopping'
        Service = 'Service'
        Stressed = 'Stressed'
        NonRecover = 'NonRecover'
        No_Contact = 'No Contact'
        Lost_Comm = 'Lost Comm'
        Stopped = 'Stopped'

    class HealthState(object):
        Unknown = pywbem.Uint16(0)
        OK = pywbem.Uint16(5)
        Degraded_Warning = pywbem.Uint16(10)
        Minor_failure = pywbem.Uint16(15)
        Major_failure = pywbem.Uint16(20)
        Critical_failure = pywbem.Uint16(25)
        Non_recoverable_error = pywbem.Uint16(30)
        # DMTF_Reserved = ..
        # Vendor_Specific = 32768..65535

    class Classifications(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Driver = pywbem.Uint16(2)
        Configuration_Software = pywbem.Uint16(3)
        Application_Software = pywbem.Uint16(4)
        Instrumentation = pywbem.Uint16(5)
        Firmware_BIOS = pywbem.Uint16(6)
        Diagnostic_Software = pywbem.Uint16(7)
        Operating_System = pywbem.Uint16(8)
        Middleware = pywbem.Uint16(9)
        Firmware = pywbem.Uint16(10)
        BIOS_FCode = pywbem.Uint16(11)
        Support_Service_Pack = pywbem.Uint16(12)
        Software_Bundle = pywbem.Uint16(13)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..0xFFFF

    class ExtendedResourceType(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Not_Applicable = pywbem.Uint16(2)
        Linux_RPM = pywbem.Uint16(3)
        HP_UX_Depot = pywbem.Uint16(4)
        Windows_MSI = pywbem.Uint16(5)
        Solaris_Package = pywbem.Uint16(6)
        Macintosh_Disk_Image = pywbem.Uint16(7)
        Debian_linux_Package = pywbem.Uint16(8)
        VMware_vSphere_Installation_Bundle = pywbem.Uint16(9)
        VMware_Software_Bulletin = pywbem.Uint16(10)
        HP_Smart_Component = pywbem.Uint16(11)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..

    class CommunicationStatus(object):
        Unknown = pywbem.Uint16(0)
        Not_Available = pywbem.Uint16(1)
        Communication_OK = pywbem.Uint16(2)
        Lost_Communication = pywbem.Uint16(3)
        No_Contact = pywbem.Uint16(4)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..

    class OperationalStatus(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        OK = pywbem.Uint16(2)
        Degraded = pywbem.Uint16(3)
        Stressed = pywbem.Uint16(4)
        Predictive_Failure = pywbem.Uint16(5)
        Error = pywbem.Uint16(6)
        Non_Recoverable_Error = pywbem.Uint16(7)
        Starting = pywbem.Uint16(8)
        Stopping = pywbem.Uint16(9)
        Stopped = pywbem.Uint16(10)
        In_Service = pywbem.Uint16(11)
        No_Contact = pywbem.Uint16(12)
        Lost_Communication = pywbem.Uint16(13)
        Aborted = pywbem.Uint16(14)
        Dormant = pywbem.Uint16(15)
        Supporting_Entity_in_Error = pywbem.Uint16(16)
        Completed = pywbem.Uint16(17)
        Power_Mode = pywbem.Uint16(18)
        Relocating = pywbem.Uint16(19)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..

    class OperatingStatus(object):
        Unknown = pywbem.Uint16(0)
        Not_Available = pywbem.Uint16(1)
        Servicing = pywbem.Uint16(2)
        Starting = pywbem.Uint16(3)
        Stopping = pywbem.Uint16(4)
        Stopped = pywbem.Uint16(5)
        Aborted = pywbem.Uint16(6)
        Dormant = pywbem.Uint16(7)
        Completed = pywbem.Uint16(8)
        Migrating = pywbem.Uint16(9)
        Emigrating = pywbem.Uint16(10)
        Immigrating = pywbem.Uint16(11)
        Snapshotting = pywbem.Uint16(12)
        Shutting_Down = pywbem.Uint16(13)
        In_Test = pywbem.Uint16(14)
        Transitioning = pywbem.Uint16(15)
        In_Service = pywbem.Uint16(16)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..

    class PrimaryStatus(object):
        Unknown = pywbem.Uint16(0)
        OK = pywbem.Uint16(1)
        Degraded = pywbem.Uint16(2)
        Error = pywbem.Uint16(3)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..


@cmpi_logging.trace_function
def object_path2nevra(op, with_epoch='NOT_ZERO'):
    """Get nevra out of object path. Also checks for validity."""
    if not isinstance(op, pywbem.CIMInstanceName):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "op must be an instance of CIMInstanceName")

    if (not "InstanceID" in op or not op['InstanceID']):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER, "Wrong keys.")
    instid = op['InstanceID']
    if not instid.lower().startswith("lmi:lmi_softwareidentity:"):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "InstanceID must start with LMI:LMI_SoftwareIdentity: prefix.")
    instid = instid[len("LMI:LMI_SoftwareIdentity:"):]
    match = util.RE_NEVRA_OPT_EPOCH.match(instid)
    if not match:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Wrong InstanceID. Expected valid nevra"
                ' (name-[epoch:]version-release.arch): "%s".' %
                instid)
    epoch = match.group('epoch')
    if not epoch:
        epoch = "0"
    return util.make_nevra(
            match.group('name'),
            epoch,
            match.group('version'),
            match.group('release'),
            match.group('arch'), with_epoch=with_epoch)

@cmpi_logging.trace_function
def object_path2pkg(op,
        kind='installed',
        include_repos=None,
        exclude_repos=None,
        repoid=None,
        return_all=False):
    """
    @param op must contain precise information of package,
    otherwise an error is raised
    @param kind one of yumdb.jobs.YumGetPackageList.SUPPORTED_KINDS
    says, where to look for given package
    @param repoid if not None, specifies repoid filter on package;
    note, that this does not make sure, that repoid will be enabled.
    @param return_all if True, return list of matching packages as returned
    by YumDB.filter_packages(), otherwise single package is returned
    """
    if not isinstance(kind, basestring):
        raise TypeError("kind must be a string")

    pkglist = YumDB.get_instance().filter_packages(kind,
            allow_duplicates=kind not in ('installed', 'avail_reinst'),
            include_repos=include_repos,
            exclude_repos=exclude_repos,
            repoid=repoid,
            nevra=object_path2nevra(op, 'ALWAYS'))
    if return_all is True:
        return pkglist
    if len(pkglist) > 0:
        return pkglist[0]
    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
            'No matching package found for InstanceID=\"%s\".' %
            op["InstanceID"])

@cmpi_logging.trace_function
def pkg2model(pkg, keys_only=True, model=None):
    """
    @param pkg can either be an instance of PackageInfo or nevra as string
    @param model if not None, will be filled with data, otherwise
    a new instance of CIMInstance or CIMObjectPath is created
    """
    if not isinstance(pkg, (basestring, PackageInfo)):
        raise TypeError("pkg must be an instance of PackageInfo or nevra")
    if isinstance(pkg, basestring) and not keys_only:
        raise ValueError("can not create instance out of nevra")
    if model is None:
        model = util.new_instance_name("LMI_SoftwareIdentity")
        if not keys_only:
            model = pywbem.CIMInstance("LMI_SoftwareIdentity", path=model)
    nevra = pkg if isinstance(pkg, basestring) else pkg.nevra
    model['InstanceID'] = 'LMI:LMI_SoftwareIdentity:'+nevra
    if not keys_only:
        model.path['InstanceID'] = model['InstanceID']  #pylint: disable=E1103
        model['Caption'] = pkg.summary
        model['Classifications'] = [pywbem.Uint16(0)]
        model['Description'] = pkg.description
        model['ElementName'] = pkg.nevra
        if pkg.installed:
            model['InstallDate'] = pywbem.CIMDateTime(pkg.install_time)
        else:
            model['InstallDate'] = pywbem.CIMProperty(
                    'InstallDate', None, type='datetime')
        model['IsEntity'] = True
        model['Name'] = pkg.name
        try:
            model["Epoch"] = pywbem.Uint32(int(pkg.epoch))
        except ValueError:
            LOG().error('Could not convert epoch "%s"'
                ' to integer for package \"%s\"!' % (pkg.epoch, pkg))
            model["Epoch"] = pywbem.CIMProperty('Epoch', None, type='uint32')
        model['Version'] = pkg.version
        model['Release'] = pkg.release
        model['Architecture'] = pkg.arch
        model['TargetTypes'] = ['rpm', 'yum']
        model['VersionString'] = pkg.evra
    return model




#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_SoftwareIdentityChecks`` provider.
"""

import os
import pywbem
import subprocess
import unittest

from lmi.test.lmibase import enable_lmi_exceptions
import package
import swbase
import util

SOFTWARE_ELEMENT_STATE_EXECUTABLE = 2

class TestSoftwareIdentityChecks(swbase.SwTestCase):
    """
    Basic cim operations test.
    """

    CLASS_NAME = "LMI_SoftwareIdentityChecks"
    KEYS = ("Check", "Element")

    def make_op(self, pkg, filepath):
        """
        :returns: Object path of ``LMI_SoftwareIdentityChecks``
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return self.cim_class.new_instance_name({
            "Check" : self.ns.LMI_SoftwareIdentityFileCheck.new_instance_name({
                "CheckID" : 'LMI:LMI_SoftwareIdentityFileCheck',
                "Name" : filepath,
                "SoftwareElementID" : pkg.nevra,
                "SoftwareElementState" : SOFTWARE_ELEMENT_STATE_EXECUTABLE,
                "TargetOperatingSystem" : util.get_target_operating_system(),
                "Version" : pkg.evra
                }),
            "Element" : self.ns.LMI_SoftwareIdentity.new_instance_name({
                "InstanceID" : 'LMI:LMI_SoftwareIdentity:' + pkg.nevra
                })
            })

    def setUp(self):
        to_uninstall = set()
        for repo in self.repodb.values():
            for pkg in repo.packages:
                to_uninstall.add(pkg.name)
        to_uninstall = list(package.filter_installed_packages(to_uninstall))
        package.remove_pkgs(to_uninstall, suppress_stderr=True)

    @swbase.test_with_packages('stable#pkg1')
    def test_get_instance(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityChecks``.
        """
        pkg = self.get_repo('stable')['pkg1']
        count = 0
        for filepath in pkg:
            objpath = self.make_op(pkg, filepath)
            inst = objpath.to_instance()
            self.assertNotEqual(inst, None,
                    "failed to get instance for %s:%s" % (pkg, filepath))
            self.assertCIMNameEqual(inst.path, objpath)
            for key in self.KEYS:
                self.assertCIMNameEqual(
                        getattr(inst, key), getattr(inst.path, key))
            count += 1
        self.assertEqual(count, 7, "there are 7 files in pkg1")

    @enable_lmi_exceptions
    @swbase.test_with_packages('stable#pkg1')
    def test_get_instance_invalid(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentityChecks``
        on not installed file.
        """
        pkg = self.get_repo('stable')['pkg1']
        objpath = self.make_op(pkg, '/not-installed-file')
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath.to_instance)

    @enable_lmi_exceptions
    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames()`` call on ``LMI_SoftwareIdentityChecks``.
        Should not be supported.
        """
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_SUPPORTED,
                self.cim_class.instance_names)

    @swbase.test_with_packages('stable#pkg3')
    def test_package_file_checks(self):
        """
        Try to get file checks associated with package.
        """
        pkg = self.get_repo('stable')['pkg3']
        filepath = '/usr/share/openlmi-sw-test-pkg3'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.Element.to_instance()
        self.assertNotEqual(inst, None,
                "get instance on %s:%s succeeds" % (pkg, filepath))
        refs = inst.associators(
                AssocClass=self.CLASS_NAME,
                Role="Element",
                ResultRole="Check",
                ResultClass="LMI_SoftwareIdentityFileCheck")
        self.assertEqual(len(refs), len(pkg))
        self.assertEqual(len(refs), 10, "there are 10 files in %s" % pkg.name)
        pkg_names = set(pkg.files)
        for ref in refs:
            self.assertIn(ref.Name, set(r.Name for r in refs))
            self.assertEqual(sorted(ref.path.key_properties()),
                ["CheckID", "Name", "SoftwareElementID",
                    "SoftwareElementState",
                    "TargetOperatingSystem", "Version"])
            self.assertEqual(ref.SoftwareElementID, pkg.nevra)
            self.assertIn(ref.Name, pkg_names)
            self.assertEqual(ref.FailedFlags, [],
                    "FailedFlags are empty for unmodified file %s:%s"
                    % (pkg, filepath))
            pkg_names.remove(ref.Name)
        self.assertEqual(len(pkg_names), 0)

    @swbase.test_with_packages('stable#pkg3')
    def test_package_file_check_names(self):
        """
        Try to get file check names associated with package.
        """
        pkg = self.get_repo('stable')['pkg3']
        objpath = self.make_op(pkg, '/usr/share/openlmi-sw-test-pkg3')
        refs = objpath.Element.to_instance().associator_names(
                AssocClass=self.CLASS_NAME,
                Role="Element",
                ResultRole="Check",
                ResultClass="LMI_SoftwareIdentityFileCheck")
        self.assertEqual(len(refs), len(pkg))
        self.assertEqual(len(refs), 10, "there are 10 files in %s" % pkg.name)
        pkg_names = set(pkg.files)
        for ref in refs:
            self.assertIn(ref.Name, set(r.Name for r in refs))
            self.assertEqual(sorted(ref.key_properties()),
                ["CheckID", "Name", "SoftwareElementID",
                    "SoftwareElementState",
                    "TargetOperatingSystem", "Version"])
            self.assertEqual(ref.SoftwareElementID, pkg.nevra)
            self.assertIn(ref.Name, pkg_names)
            pkg_names.remove(ref.Name)
        self.assertEqual(len(pkg_names), 0)

    @swbase.test_with_packages('stable#pkg3')
    def test_file_check_package(self):
        """
        Try to get package assocatied to file check.
        """
        pkg = self.get_repo('stable')['pkg3']
        filepath = '/usr/share/openlmi-sw-test-pkg3/README'
        objpath = self.make_op(pkg, filepath)
        inst = objpath.Check.to_instance()
        self.assertNotEqual(inst, None,
                "get instance for %s:%s succeeds" % (pkg, filepath))
        refs = inst.associators(
                AssocClass=self.CLASS_NAME,
                Role="Check",
                ResultRole="Element",
                ResultClass="LMI_SoftwareIdentity")
        self.assertEqual(len(refs), 1)
        ref = refs[0]
        self.assertCIMNameEqual(ref.path, objpath.Element)
        self.assertEqual(ref.ElementName, pkg.nevra)
        self.assertEqual(ref.VersionString, pkg.evra)

        # try with removed file
        os.remove(filepath)
        refs = objpath.Check.to_instance().associators(
                AssocClass=self.CLASS_NAME,
                Role="Check",
                ResultRole="Element",
                ResultClass="LMI_SoftwareIdentity")
        self.assertEqual(len(refs), 1)

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestSoftwareIdentityChecks)

if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""Pylint plugin to check OpenLMI unittest modules.""" 

from logilab.astng import scoped_nodes
from pylint.interfaces import IASTNGChecker
from pylint.checkers import BaseChecker
import unittest

def is_test_case_method(node):
    """
    Check, whether node method is defined by TestCase base class.
    """
    if (   isinstance(node, scoped_nodes.Function)
       and node.name in unittest.TestCase.__dict__):
        return True
    return False

def is_test_case_subclass(node):
    """
    Check, whether node is a subclass of TestCase base class.
    """
    if not isinstance(node, scoped_nodes.Class):
        return False
    for ancestor in node.ancestors():
        if (   ancestor.name == 'TestCase'
           and ancestor.getattr('__module__')[0].value.startswith('unittest')):
            return True
    return False

class TestCaseChecker(BaseChecker):
    """
    Checker for OpenLMI unittest modules.
    Right now it just suppresses unwanted messages.
    """

    __implements__ = IASTNGChecker

    name = 'unittest_case'
    # When we do have some real warning messages/reports,
    # remote W9920. We need it just to get registered.
    msgs = { 'W9920': ('Dummy', "This is a dummy message.")}

    priority = -1

    def visit_class(self, node):
        """
        Suppress Naming and 'Attribute creation out of __init__'
        errors for every TestCase subclass.
        """
        if not is_test_case_subclass(node):
            return
        if 'setUp' in node:
            self.linter.disable('W0201',
                    scope='module', line=node['setUp'].lineno)
        for child in node.get_children():
            if not is_test_case_method(child):
                continue
            self.linter.disable('C0103',
                    scope='module', line=child.lineno)

def register(linter):
    """required method to auto register our checker"""
    linter.register_checker(TestCaseChecker(linter))




# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Tomas Bzatek <tbzatek@redhat.com>
#

from journald_common import JournalBase
import time
import syslog
import unittest

class TestIndications(JournalBase):
    """
    Class for testing LMI_JournalMessageLog indications
    """

    NEEDS_INDICATIONS = True

    def test_check_good_filter(self):
        """
        Journal: Test good indication filter
        """
        filter_name = "test_good_filter_%d" % (time.time() * 1000)
        sub = self.subscribe(filter_name, "select * from LMI_JournalLogRecordInstanceCreationIndication where SourceInstance isa LMI_JournalLogRecord")
        self.assertIsNotNone(sub)
        self.unsubscribe(filter_name);


    def test_message_send(self):
        """
        Journal: Test message logging and its retrieval from journal
        """
        filter_name = "test_message_send_%d" % (time.time() * 1000)
        syslog_msg = "== LMI_Journald test message =="
        sub = self.subscribe(filter_name, "select * from LMI_JournalLogRecordInstanceCreationIndication where SourceInstance isa LMI_JournalLogRecord")
        syslog.syslog(syslog_msg)
        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_JournalLogRecordInstanceCreationIndication")
        self.assertIn("SourceInstance", indication.keys())
        self.assertTrue(indication["SourceInstance"] is not None)
        self.assertIn(syslog_msg, indication["SourceInstance"]["DataFormat"])
        self.unsubscribe(filter_name);

if __name__ == '__main__':
    unittest.main()
